/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: NUTRIENTE_PRATO

Descrio:
Funo para calcular e retornar os Nutrientes dos pratos.

P_FILIAL     - Codigo da Filial.
P_SERVICO    - String contendo sequencia de Servios a serem calculados.
P_NUTRIENTE  - String contendo Nutrientes a serem calculados.
P_IDNDPCAL   - Indicador de calculo do NDPCal.
P_CDPRATOINI - Codigo do prato inicial.
P_CDPRATOFIN - Codigo do prato final.


****************************************************************
             CLCULO DO NDPCAL
****************************************************************
A frmula usada para clculo do NDPCal% :

NDPCAL% = NPU * Sum(Protenas) * 4
          ------------------------
                    VCT
onde:

NPU = Somatrio das protenas dos componentes da receita multiplicados
      pelo Fator de Clculo de NDPCAL, informado na tela de parametrizao
      por grupo de produtos, sendo 0,7 para alimentos de origem animal;
      0,6 para leguminosas e 0,5 para cereais, frutas e outros vegetais.
Sum(Protenas) = Somatrio das protenas dos componentes da receita
VCT = "Valor Calrico Total" = Soma(carboidratos) x 4 +
                               Soma(lipdeos)    x 9  +
                               Soma(protenas)   x 4  +
                               Soma(lcoois)     x 7
****************************************************************
*/

DROP FUNCTION NUTRIENTE_PRATO
/

DROP TYPE NUTRIENTE_PRATO_TABLE
/

DROP TYPE NUTRIENTE_PRATO_TYPE
/

CREATE TYPE NUTRIENTE_PRATO_TYPE AS OBJECT(
   CDFILINUTR VARCHAR2(04),
   CDSERVNUTR VARCHAR2(05),
   CDFAPRATNUTR VARCHAR2(04),
   CDPRATNUTR VARCHAR2(10),
   CDPRODNUTR VARCHAR2(10),
   CDNUTRCALC VARCHAR2(03),
   QTPRODNUTR FLOAT,
   QTNUTRCALC FLOAT,
   QTPROTLIQU FLOAT,
   VRCALORNUTR FLOAT,
   VRPECONSNUTR FLOAT
   )
/

CREATE TYPE NUTRIENTE_PRATO_TABLE AS TABLE OF NUTRIENTE_PRATO_TYPE
/

CREATE OR REPLACE
FUNCTION NUTRIENTE_PRATO(
   P_FILIAL     IN VARCHAR2,
   P_SERVICO    IN VARCHAR2,
   P_NUTRIENTE  IN VARCHAR2,
   P_IDNDPCAL   IN VARCHAR2,
   P_CDPRATOINI IN VARCHAR2,
   P_CDPRATOFIN IN VARCHAR2
   ) RETURN NUTRIENTE_PRATO_TABLE PIPELINED IS

   OUT_RECORD NUTRIENTE_PRATO_TYPE := NUTRIENTE_PRATO_TYPE(
                                      NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
   V_CDPROTEINA FLOAT;
   V_CDCALORIAS FLOAT;
   V_QTPROTLIQU FLOAT;
   V_VRCALORNUTR FLOAT;
BEGIN
   SELECT MAX(CDPROTEINA)
          INTO V_CDPROTEINA
   FROM PARAMGERAL;

   SELECT MAX(CDCALORIAS)
          INTO V_CDCALORIAS
   FROM PARAMGERAL;

   FOR C_NUTRIENTE IN(
      SELECT SE.CDFILIAL, SE.CDSERVFILI, P.CDFILAUXPRAT, P.CDPRATO, R.CDPRODUTO,
           NP.CDNUTRIENTE, R.QTCALCNUTR AS QTPRODUTO, R.VRPEFATNUT AS VRPEFATNUT,
           NP.QTNUTRPROD * 10 * NVL(DECODE(PRODFILI.VRPESOUNIDFI, 0, R.VRPESOUNID, PRODFILI.VRPESOUNIDFI),R.VRPESOUNID) * R.QTCALCNUTR  AS QTCALCNUTR
      FROM SERVICO SE, PRATO P, 
           TABLE (RECEITA_UTIL(P_FILIAL, SE.CDSERVFILI, P.CDPRATO, P.CDFILAUXPRAT,'0', 'Z', P.CDPRATO,0,0,'N') ) R, 
           NUTRPROD NP, PRODFILI
      WHERE (SE.CDFILIAL = P_FILIAL)                                  
        AND (INSTR(P_SERVICO, SE.CDSERVFILI) <> 0)
        AND (P.CDPRATO BETWEEN P_CDPRATOINI AND P_CDPRATOFIN)
        AND (P.CDFILAUXPRAT = 'P' OR P.CDFILAUXPRAT = SE.CDFILIAL)
        AND (R.CDPRATOPAI = P.CDPRATO)
        AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
        AND (NP.CDPRODUTO = R.CDPRODUTO)
        AND ((INSTR(P_NUTRIENTE, NP.CDNUTRIENTE) <> 0) OR (P_NUTRIENTE IS NULL))
        AND (PRODFILI.CDFILIAL = SE.CDFILIAL)
        AND (PRODFILI.CDPRODUTO = R.CDPRODUTO)
   ) LOOP

      V_QTPROTLIQU := 0;
      V_VRCALORNUTR := 0;
      /* --- CALCULO DO DNPCal --- */
      IF P_IDNDPCAL = 'S' THEN
         IF V_CDPROTEINA = C_NUTRIENTE.CDNUTRIENTE THEN
            /* ---GRAVA O VALOR DE PROTEINA LIQUIDA (NPU) -- */
            SELECT NVL(MAX(VRFATCALNDPC),0)
                   INTO V_QTPROTLIQU
            FROM NDPCAL
            WHERE  CDPRODUTO = (SELECT MAX(CDPRODUTO)
                                FROM NDPCAL
                                WHERE INSTR(C_NUTRIENTE.CDPRODUTO,CDPRODUTO) = 1
                                );
            V_QTPROTLIQU := V_QTPROTLIQU * C_NUTRIENTE.QTCALCNUTR;
         END IF;
         /* --------------------- GRAVA O VALOR DE CALORIAS -------------------- */
         IF V_CDCALORIAS = C_NUTRIENTE.CDNUTRIENTE THEN
            V_VRCALORNUTR := C_NUTRIENTE.QTCALCNUTR;
         END IF;
      END IF;

      OUT_RECORD.CDFILINUTR := C_NUTRIENTE.CDFILIAL;
      OUT_RECORD.CDSERVNUTR := C_NUTRIENTE.CDSERVFILI;
      OUT_RECORD.CDFAPRATNUTR := C_NUTRIENTE.CDFILAUXPRAT;
      OUT_RECORD.CDPRATNUTR := C_NUTRIENTE.CDPRATO;
      OUT_RECORD.CDPRODNUTR := C_NUTRIENTE.CDPRODUTO;
      OUT_RECORD.CDNUTRCALC := C_NUTRIENTE.CDNUTRIENTE;
      OUT_RECORD.QTPRODNUTR := ROUND(C_NUTRIENTE.QTPRODUTO, 5);
      OUT_RECORD.QTNUTRCALC := ROUND(C_NUTRIENTE.QTCALCNUTR, 5);
      OUT_RECORD.QTPROTLIQU := ROUND(V_QTPROTLIQU, 5);
      OUT_RECORD.VRCALORNUTR := ROUND(V_VRCALORNUTR, 5);
      OUT_RECORD.VRPECONSNUTR := C_NUTRIENTE.VRPEFATNUT;
      PIPE ROW (OUT_RECORD);

   END LOOP;
   RETURN;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('NUTRIENTE_PRATO','6.25.xxx'); END;
/
